# coding=utf-8
from __future__ import absolute_import, division, print_function, unicode_literals

import octoprint.plugin
from octoprint.util import RepeatedTimer
import os
import flask


class PrnBaseStatusPlugin(octoprint.plugin.StartupPlugin,
							octoprint.plugin.SimpleApiPlugin,
							octoprint.plugin.AssetPlugin,
							octoprint.plugin.TemplatePlugin):

	def __init__(self):
		self.updatetimer = None
		self.tiTicks=0
		self.TIMERTIME_SEC=15
		self.KEY_REQ_CGRU_LABEL = "REQ_CGRU_LABEL"
		self.KEY_REQ_CVAL_LABEL = "REQ_CVAL_LABEL"
		self.REQ_COMMAND = "CAMUSRCTRL"
		self.SCMD_PRN_ON = "gpio_leo_on"
		self.SCMD_PRN_OFF = "gpio_leo_off"
		self.SCMD_PWR_ON = "gpio_prnpwr_on"
		self.SCMD_PWR_OFF = "gpio_prnpwr_off"
		self.GPIN_PWR = 21
		self.GPIN_PRN = 12
		self.CMDDEST_PWR = 1
		self.CMDDEST_PRN = 2
		self.CMD_ON = 1
		self.CMD_OFF = 0
		self.electricOn = False
		self.prnPower = False

	def initialize(self):
		self._logger.info( "initializing" )

	def on_after_startup(self):
		self.startTimer()

	def get_assets(self):
		return { "js": ["js/PrnBaseStatus.js"], }

	def get_template_configs(self):
		return dict(type="navbar", custom_bindings=True)

	def get_api_commands(self):
		self._logger.info( "get_api_commands called" )
		return dict( LEOONOFF=[], command2=["abcde"] )

	def on_api_command(self, command, data):
		self._logger.info( "on_api_command triggered " + command)
		self._logger.info( data )
		inDataDict = data
		inCmdGrp = inDataDict.get(self.KEY_REQ_CGRU_LABEL)
		inCmd = inDataDict.get(self.KEY_REQ_CVAL_LABEL)
		if( (inCmd is None) or (inCmd < 0) or (inCmd > 1)):
			self._logger.error( "CVAL invalid value")
			return
		else:
			self._logger.info( "CVAL " + str(inCmd))
			if( (inCmdGrp is None) or (inCmdGrp < 1) or (inCmdGrp > 2)):
				self._logger.info( "CGRU invalid value")
				return
			else:
				self._logger.info( "CGRU " + str(inCmdGrp))
				if(inCmdGrp == self.CMDDEST_PWR):
					if(inCmd == self.CMD_OFF):
						self.doOsCmd(self.SCMD_PRN_OFF)
						self.doOsCmd(self.SCMD_PWR_OFF)
					else:
						self.doOsCmd(self.SCMD_PWR_ON)
				else:
					if(inCmd == self.CMD_OFF):
						self.doOsCmd(self.SCMD_PRN_OFF)
					else:
						self.doOsCmd(self.SCMD_PRN_ON)
		self.timerTick()

	def on_api_get(self, request):
		self._logger.info( "on_api_get called " + request )
# return flask.jsonify(foo="bar")

#internal
	def send_updateUI(self):
		payload = {"power": self.electricOn, "printer": self.prnPower}
		self._plugin_manager.send_plugin_message("PrnBaseStatus", payload)
		self._logger.info( "send update. powerOn:" + str(self.electricOn) + " printerOn:" + str(self.prnPower))

	def startTimer(self):
		if(self.updatetimer != None):
			self._logger.info("Stop timer")
			self.updatetimer.cancel()
		self.tiTicks = 0
		self._logger.info("Start timer")
		self.updatetimer = RepeatedTimer(self.TIMERTIME_SEC,self.timerTick, run_first=False)
		self.updatetimer.start()

	def stopTimer(self):
		if(self.updatetimer != None):
			self._logger.info("Stop timer")
			self.updatetimer.cancel()
			self.updatetimer = None

	def timerTick(self):
		self._logger.info("timer tick")
		self.readState()
		self.send_updateUI()

	def readState(self):
		gvpath = "/sys/class/gpio/gpio" + str(self.GPIN_PWR) + "/value"
		try:
			pinfd = open(gvpath, 'r')
			fdstateByte = pinfd.read(1)
			pinfd.close()
			self._logger.info( gvpath + " state is " + fdstateByte )
			if( fdstateByte == '0' ):
				self.electricOn = True
			else:
				self.electricOn = False
		except Exception as e:
			self._logger.error("could not open " + gvpath + " error:" + str(e))
		gvpath = "/sys/class/gpio/gpio" + str(self.GPIN_PRN) + "/value"
		try:
			pinfd = open(gvpath, 'r')
			fdstateByte = pinfd.read(1)
			pinfd.close()
			self._logger.info( gvpath + " state is " + fdstateByte )
			if( fdstateByte == '0' ):
				self.prnPower = True
			else:
				self.prnPower = False
		except Exception as e:
			self._logger.error("could not open " + gvpath +  " error:" + str(e))

	def doOsCmd(self, syscall):
		if( 0 == os.system( syscall ) ):
			self._logger.info(syscall + "successful")
		else:
			self._logger.error(syscall + "failed")

			
__plugin_name__ = "Printer Base Status"
__plugin_pythoncompat__ = ">=2.7,<4"


def __plugin_load__():
	global __plugin_implementation__
	__plugin_implementation__ = PrnBaseStatusPlugin()

